﻿#if UNITY_EDITOR

/*==========================================================================*/
/*		Using																*/
/*==========================================================================*/

using System;
using System.Collections.Generic;
using System.IO;
using TogakiMaker.Core.EditorSettings;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using UnityEditor;
using UnityEngine;

/*==========================================================================*/
/*		class																*/
/*==========================================================================*/
namespace TogakiMaker.Core.MainTool.EditorScripts.DataImport
{
	/*==========================================================================*/
	/*!
		@brief	Excelフォーマット専用のストーリーインポートクラス
	*/
	/*==========================================================================*/
	public class DataImporterFromExcel : IDataImporter
	{
		/*==========================================================================*/
		/*		var																	*/
		/*==========================================================================*/
		public string ImporterName => "Import from Excel";
		
		private string lastAccessPath;
		
		/*==========================================================================*/
		/*		func																*/
		/*==========================================================================*/
		
		public bool IsSyncImportSupport(
			string importFilePath
		)
		{
			return Path.GetExtension(importFilePath) == ".xls";
		}
		
		public ImportDataFormat DataImport(
			string importFilePath
		)
		{
			if (string.IsNullOrEmpty(importFilePath) )
			{
				return null;
			}
			
			lastAccessPath = importFilePath;

			if (Path.GetExtension(importFilePath) != ".xls")
			{
				return null;
			}
			
			//----
			
			
			try
			{
				var allSheetImportedData = new ImportDataFormat();
				
				using (var fileStream = new FileStream(
						importFilePath,
						FileMode.Open,
						System.IO.FileAccess.Read,
						FileShare.ReadWrite
					)
				)
				{
					Debug.Log( importFilePath );
					
					
					var book = (IWorkbook)null;
					if (Path.GetExtension(importFilePath) == ".xls")
					{
						book = new HSSFWorkbook(fileStream);
					}
					else
					{
						book = new XSSFWorkbook(fileStream);
					}
					
					for (var sheetCt = 0; sheetCt < book.NumberOfSheets; sheetCt++)
					{
						var sheet = book.GetSheetAt(sheetCt);
						if (sheet == null)
							continue;
						
						var importDataFormat = new ImportDataFormat();
						
						// Parse the first line of text as the header information.
						// 1行目の文字列を読み込み、ヘッダー情報として解析する。
						{
							var variableNames = new List<string>();
							var titleRow = sheet.GetRow(0);
							for (var i = 0; i < titleRow.LastCellNum; i++)
							{
								variableNames.Add(titleRow.GetCell(i).StringCellValue);
							}
							
							importDataFormat.SetVariableNames( variableNames );
						}
						
						// 各行を読み込み
						for (var i = 1; i <= sheet.LastRowNum; i++)
						{
							var dataRow = sheet.GetRow(i);
							if (dataRow == null)
								continue;
						
							// どの言語でも本文がないものはスキップ
							if (dataRow.LastCellNum <= 2)
								continue;
						
							// コメント行をスキップ
							var idCellValue = dataRow.GetCell(0)?.StringCellValue;
							if (string.IsNullOrEmpty(idCellValue) ||
							    idCellValue.IndexOf("//") == 0
							)	continue;
							
							// 行内のデータを登録
							var rowDataLise = new List<string>(); 
							for (var j = 0; j < importDataFormat.GetVariableNamesCount(); j++)
							{
								rowDataLise.Add( dataRow.GetCell(j)?.StringCellValue );
							}
							importDataFormat.AddRowValues(rowDataLise);
						}
						
						// 全てのシートの情報を統合
						allSheetImportedData.Marge( importDataFormat );
					}
					
				}

				return allSheetImportedData;
			}
			catch (Exception ex)
			{
				Debug.LogError( ex.Message );
				return null;
			}
		}
		
		public void AsyncDataImport(
			Action<ImportDataFormat> onDataImported,
			Action<string> onCancel
		)
		{
			var accessPath = 
				string.IsNullOrEmpty(lastAccessPath) ? 
					TogakiMakerEditorSettings.CreateOrGetEditInstance().StoryFileImportDefaultPath :
					lastAccessPath;

			
			var importFilePath = EditorUtility.OpenFilePanel(
				"open data file",
				accessPath,
				"xls");

			AsyncDataImport( importFilePath , onDataImported , onCancel );
		}

		public void AsyncDataImport(
			string importFilePath,
			Action<ImportDataFormat> onDataImported,
			Action<string> onCancel
		)
		{
			var importedData = DataImport(importFilePath);
			if (importedData == null)
			{
				onCancel?.Invoke(null);
				return;
			}
			onDataImported?.Invoke(importedData);
		}
	}
}

#endif
